@@ -27,7 +27,9 @@
 #define CURSOR_SIZE_KEY "X-GNOME-Metatheme/CursorSize"
 #define SOUND_THEME_KEY "X-GNOME-Metatheme/SoundTheme"
 #define APPLICATION_FONT_KEY "X-GNOME-Metatheme/ApplicationFont"
+#define DOCUMENTS_FONT_KEY "X-GNOME-Metatheme/DocumentsFont"
 #define DESKTOP_FONT_KEY "X-GNOME-Metatheme/DesktopFont"
+#define WINDOWTITLE_FONT_KEY "X-GNOME-Metatheme/WindowTitleFont"
 #define MONOSPACE_FONT_KEY "X-GNOME-Metatheme/MonospaceFont"
 #define BACKGROUND_IMAGE_KEY "X-GNOME-Metatheme/BackgroundImage"
 
@@ -342,10 +344,18 @@ gnome_theme_read_meta_theme (GnomeVFSURI *meta_theme_uri)
   if (str != NULL)
     meta_theme_info->application_font = g_strdup (str);
 
+  str = gnome_desktop_item_get_string (meta_theme_ditem, DOCUMENTS_FONT_KEY);
+  if (str != NULL)
+    meta_theme_info->documents_font = g_strdup (str);
+
   str = gnome_desktop_item_get_string (meta_theme_ditem, DESKTOP_FONT_KEY);
   if (str != NULL)
     meta_theme_info->desktop_font = g_strdup (str);
 
+  str = gnome_desktop_item_get_string (meta_theme_ditem, WINDOWTITLE_FONT_KEY);
+  if (str != NULL)
+    meta_theme_info->windowtitle_font = g_strdup (str);
+
   str = gnome_desktop_item_get_string (meta_theme_ditem, MONOSPACE_FONT_KEY);
   if (str != NULL)
     meta_theme_info->monospace_font = g_strdup (str);
@@ -679,8 +689,7 @@ read_cursor_fonts (void)
 
 static void
 handle_change_signal (gpointer             data,
-		      GnomeThemeChangeType change_type,
-		      GnomeThemeElement    element)
+		      GnomeThemeChangeType change_type)
 {
 #ifdef DEBUG
   gchar *type_str = NULL;
@@ -696,7 +705,7 @@ handle_change_signal (gpointer             data,
   for (list = callbacks; list; list = list->next)
   {
     ThemeCallbackData *callback_data = list->data;
-    (* callback_data->func) (theme, change_type, element, callback_data->data);
+    (* callback_data->func) (theme, change_type, callback_data->data);
   }
 
 #ifdef DEBUG
@@ -706,6 +715,15 @@ handle_change_signal (gpointer             data,
     type_str = "icon";
   else if (theme->type == GNOME_THEME_TYPE_CURSOR)
     type_str = "cursor";
+  else if (theme->type == GNOME_THEME_TYPE_REGULAR) {
+    GnomeThemeInfo *rtheme = (GnomeThemeInfo *) theme;
+    if (rtheme->has_gtk)
+      element_str = "gtk-2";
+    else if (rtheme->has_keybinding)
+      element_str = "keybinding";
+    else if (rtheme->has_metacity)
+      element_str = "metacity";
+  }
 
   if (change_type == GNOME_THEME_CHANGE_CREATED)
     change_str = "created";
@@ -714,13 +732,6 @@ handle_change_signal (gpointer             data,
   else if (change_type == GNOME_THEME_CHANGE_DELETED)
     change_str = "deleted";
 
-  if (element & GNOME_THEME_GTK_2)
-    element_str = "gtk-2";
-  else if (element & GNOME_THEME_GTK_2_KEYBINDING)
-    element_str = "keybinding";
-  if (element & GNOME_THEME_METACITY)
-    element_str = "metacity";
-
   if (type == GNOME_THEME_TYPE_REGULAR)
     {
       g_print ("theme \"%s\" has a theme of type %s (priority %d) has been %s\n",
@@ -785,7 +796,7 @@ update_theme_index (GnomeVFSURI       *index_uri,
 
 	  g_hash_table_insert (theme_hash_by_uri, g_strdup (common_theme_dir), theme_info);
 	  add_theme_to_hash_by_name (theme_hash_by_name, theme_info);
-	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED, key_element);
+	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED);
 	}
     }
   else
@@ -816,15 +827,15 @@ update_theme_index (GnomeVFSURI       *index_uri,
 
       if (theme_exists && theme_used_to_exist)
 	{
-	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED, key_element);
+	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED);
 	}
       else if (theme_exists && !theme_used_to_exist)
 	{
-	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED, key_element);
+	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED);
 	}
       else if (!theme_exists && theme_used_to_exist)
 	{
-	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_DELETED, key_element);
+	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_DELETED);
 	}
 
       if (!theme_info->has_metacity && !theme_info->has_keybinding && !theme_info->has_gtk)
@@ -929,7 +940,7 @@ update_common_theme_dir_index (GnomeVFSURI   *theme_index_uri,
 	{
 	  g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
 	  add_theme_to_hash_by_name (hash_by_name, theme_info);
-	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED, 0);
+	  handle_change_signal (theme_info, GNOME_THEME_CHANGE_CREATED);
 	}
     }
   else
@@ -943,7 +954,7 @@ update_common_theme_dir_index (GnomeVFSURI   *theme_index_uri,
    	      remove_theme_from_hash_by_name (hash_by_name, old_theme_info);
 	      g_hash_table_insert (hash_by_uri, g_strdup (common_theme_dir), theme_info);
 	      add_theme_to_hash_by_name (hash_by_name, theme_info);
-	      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED, 0);
+	      handle_change_signal (theme_info, GNOME_THEME_CHANGE_CHANGED);
 	      theme_free (old_theme_info);
 	    }
 	  else
@@ -956,7 +967,7 @@ update_common_theme_dir_index (GnomeVFSURI   *theme_index_uri,
 	  g_hash_table_remove (hash_by_uri, common_theme_dir);
    	  remove_theme_from_hash_by_name (hash_by_name, old_theme_info);
 
-	  handle_change_signal (old_theme_info, GNOME_THEME_CHANGE_DELETED, 0);
+	  handle_change_signal (old_theme_info, GNOME_THEME_CHANGE_DELETED);
 	  theme_free (old_theme_info);
 	}
     }
@@ -1698,7 +1709,9 @@ gnome_theme_meta_info_free (GnomeThemeMetaInfo *meta_theme_info)
   g_free (meta_theme_info->name);
   g_free (meta_theme_info->comment);
   g_free (meta_theme_info->application_font);
+  g_free (meta_theme_info->documents_font);
   g_free (meta_theme_info->desktop_font);
+  g_free (meta_theme_info->windowtitle_font);
   g_free (meta_theme_info->monospace_font);
   g_free (meta_theme_info->background_image);
   g_free (meta_theme_info->gtk_theme_name);
@@ -1722,7 +1735,9 @@ gnome_theme_meta_info_print (GnomeThemeMetaInfo *meta_theme_info)
   g_print ("icon_theme_name: %s\n", meta_theme_info->icon_theme_name);
   g_print ("sound_theme_name: %s\n", meta_theme_info->sound_theme_name);
   g_print ("application_font: %s\n", meta_theme_info->application_font);
+  g_print ("documents_font: %s\n", meta_theme_info->documents_font);
   g_print ("desktop_font: %s\n", meta_theme_info->desktop_font);
+  g_print ("windowtitle_font: %s\n", meta_theme_info->windowtitle_font);
   g_print ("monospace_font: %s\n", meta_theme_info->monospace_font);
   g_print ("background_image: %s\n", meta_theme_info->background_image);
 }
@@ -1787,9 +1802,15 @@ gnome_theme_meta_info_compare (GnomeThemeMetaInfo *a,
   cmp = safe_strcmp (a->application_font, b->application_font);
   if (cmp != 0) return cmp;
 
+  cmp = safe_strcmp (a->documents_font, b->documents_font);
+  if (cmp != 0) return cmp;
+
   cmp = safe_strcmp (a->desktop_font, b->desktop_font);
   if (cmp != 0) return cmp;
 
+  cmp = safe_strcmp (a->windowtitle_font, b->windowtitle_font);
+  if (cmp != 0) return cmp;
+
   cmp = safe_strcmp (a->monospace_font, b->monospace_font);
   if (cmp != 0) return cmp;
 
